/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl;

import java.util.HashMap;
import java.util.Map;

/**
 * An abstract {@link Node}.
 */

public abstract class AbstractNode implements Node {

  private String type;

  private String source;
  
  private Map decorations = new HashMap();

  public AbstractNode (final String type) {
    this.type = type;
  }
  
  // --------------------------------------------------------------------------
  // Implementation of the Node interface
  // --------------------------------------------------------------------------
  
  public String astGetType () {
    return type;
  }

  public String astGetSource () {
    return source;
  }

  public void astSetSource (final String source) {
    this.source = source;
  }
  
  public Object astGetDecoration(String name) {
    return decorations.get(name);
  }

  public Map astGetDecorations() {
    return new HashMap(decorations);
  }

  public void astSetDecoration(String name, Object decoration) {
    decorations.put(name, decoration);
  }

  public void astSetDecorations(Map decorations) {
    this.decorations.putAll(decorations);
  }

}
