/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;

import org.objectweb.deployment.scheduling.core.api.Task;

/**
 * Basic implementation of the {@link TaskMap} interface.
 */

public class BasicTaskMap extends HashMap implements TaskMap {

  private List values = new ArrayList();
  
  // --------------------------------------------------------------------------
  // Implementation of the TaskMap interface
  // --------------------------------------------------------------------------
  
  public Task[] getTasks () {
    return (Task[])values.toArray(new Task[size()]);
  }
  
  public Task getTask (final String type, final Object id)
    throws NoSuchElementException
  {
    Task t = (Task)get(new Key(type, id));
    if (t == null) {
      throw new NoSuchElementException();
    }
    return t;
  }
  
  public void addTask (final String type, final Object id, final Task task) {
    Key key = new Key(type, id);
    if (get(key) == null) {
      put(key, task);
      values.add(task);
    }
  }
  
  // --------------------------------------------------------------------------
  // Inner classes
  // --------------------------------------------------------------------------
  
  static class Key {
    
    public final String type;
    
    public final Object id;
    
    public Key (final String type, final Object id) {
      this.type = type;
      this.id = id;
    }
    
    public boolean equals (final Object o) {
      if (o instanceof Key) {
        Key k = (Key)o;
        return k.type.equals(type) && k.id.equals(id);
      }
      return false;
    }
    
    public int hashCode () {
      return type.hashCode()*id.hashCode();
    }
  }
}
