/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl;

import java.util.NoSuchElementException;

import org.objectweb.deployment.scheduling.core.api.Task;

/**
 * A map associating {@link Task}s to task identifiers and task types.
 * Two tasks of two distinct types may have the same identifier. 
 */

public interface TaskMap {

  /**
   * Returns the tasks contained in this task map.
   * 
   * @return the tasks contained in this task map.
   */
  
  Task[] getTasks ();
  
  /**
   * Returns the task of the given type and identifier.
   * 
   * @param type the type of the task that must be returned.
   * @param id the identifier of the task that must be returned.
   * @return the task whose type and identifier are given.
   * @throws NoSuchElementException if there is no such task.
   */
  
  Task getTask (String type, Object id) throws NoSuchElementException;
  
  /**
   * Adds the given task to this task map.
   * 
   * @param type the type of the task to be added.
   * @param id the identifier of the task to be added.
   * @param task the task itself.
   */
  
  void addTask (String type, Object id, Task task);
}
