/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.arguments;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.AbstractLoader;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Node;

public class ArgumentLoader extends AbstractLoader {
  
  // --------------------------------------------------------------------------
  // Implementation of the Loader interface
  // --------------------------------------------------------------------------
  
  public Definition load (final String name, final Map context) 
    throws ADLException 
  {
    Definition d = clientLoader.load(name, context);
    Map values = new HashMap();
    if (d instanceof ArgumentDefinition) {
      String args = ((ArgumentDefinition)d).getArguments();
      if (args != null) {
        StringTokenizer st = new StringTokenizer(args, ",");
        int i = 0;
        while (st.hasMoreTokens()) {
          String arg = st.nextToken();
          Object value = context.get(arg);
          if (value == null) {
            value = context.get(" arg " + i);
          }
          if (value == null) {
            throw new ADLException(
              "No value defined for argument '" + arg + "'", (Node)d, null);
          }
          values.put(arg, value);
          ++i;
        }
        ((ArgumentDefinition)d).setArguments(null);
      }
    }
    evaluate((Node)d, values);
    return d;
  }

  // --------------------------------------------------------------------------
  // Argument evaluation methods
  // --------------------------------------------------------------------------
  
  void evaluate (final Node node, final Map context) 
    throws ADLException
  {
    Map attrs = node.astGetAttributes();
    Iterator i = attrs.keySet().iterator();
    while (i.hasNext()) {
      String attr = (String)i.next();
      String value = (String)attrs.get(attr);
      if (value != null) {
        try {
          value = evaluate(value, context);
        } catch (ADLException e) {
          throw new ADLException(e.getMessage(), node, null);
        }
        attrs.put(attr, value);
      }
    }
    node.astSetAttributes(attrs);
    
    String[] nodeTypes = node.astGetNodeTypes();
    for (int j = 0; j < nodeTypes.length; ++j) {
      Node[] nodes = node.astGetNodes(nodeTypes[j]);
      for (int k = 0; k < nodes.length; ++k) {
        Node n = nodes[k];
        if (n != null) {
          evaluate(n, context);
        }
      }
    }
  }
  
  String evaluate (final String s, final Map context) throws ADLException {
    int i = s.indexOf("${");
    if (i == -1) {
      return s;
    }
    int j = s.indexOf('}', i);
    if (j == -1) {
      throw new ADLException("Malformed variable reference in '" + s + "'", null);
    }
    String arg = s.substring(i + 2, j);
    String value = (String)context.get(arg);
    if (value == null) {
      throw new ADLException("Undefined variable '" + arg + "'", null);
    }
    return evaluate(s.substring(0, i) + value + s.substring(j + 1), context);
  }
}
