/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.bindings;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * A Java based implementation of the {@link BindingBuilder} interface.
 */

public class JavaBindingBuilder implements BindingBuilder {

  /**
   * Associates a list of bindings to each context. The contexts are the last
   * argument of the bindComponent method. The list of bindings for a context
   * is the list of the primitive bindings that have been build so far in this
   * context, as a list of Binding objects.
   */
    
  private Map bindingLists = new WeakHashMap();
  
  // --------------------------------------------------------------------------
  // Implementation of the BindingBuilder interface
  // --------------------------------------------------------------------------
  
  public void bindComponent (
    final int type,
    final Object clientComp, 
    final String clientItf, 
    final Object serverComp, 
    final String serverItf, 
    final Object context) throws Exception
  {
    Binding b = new Binding();
    b.client = clientComp;
    b.clientInterface = clientItf;
    b.server = serverComp;
    b.serverInterface = serverItf;
    
    List bindings = (List)bindingLists.get(context);
    if (bindings == null) {
      bindings = new ArrayList();
      bindingLists.put(context, bindings);
    }
    bindings.add(b);
        
    for (int i = 0; i < bindings.size(); ++i) {
      b = (Binding)bindings.get(i);
      Object client = b.client;
      String clientInterface = b.clientInterface;
      Object server = b.server;
      String serverInterface = b.serverInterface;
      top: while (server != null && server instanceof Map) {
        for (int j = 0; j < bindings.size(); ++j) {
          b = (Binding)bindings.get(j);
          if (b.client == server && b.clientInterface.equals(serverInterface)) {
            server = b.server;
            serverInterface = b.serverInterface;
            continue top;
          }
        }
        server = null;
      }
      if (server != null) {
        if (client instanceof Map) {
          ((Map)client).put(clientInterface, server);
        } else {
          Method m = client.getClass().getMethod("bindFc", new Class[] {
            String.class, Object.class
          });
          m.invoke(client, new Object[] { clientInterface, server });
          bindings.remove(i--);
        }
      }
    }
  }  
  
  // --------------------------------------------------------------------------
  // Inner classes
  // --------------------------------------------------------------------------
  
  static class Binding {
    Object client;
    String clientInterface;
    Object server;
    String serverInterface;
  }
}
