/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.conform;

import java.util.Map;
import java.util.HashMap;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Loader;
import org.objectweb.fractal.adl.arguments.ArgumentLoader;
import org.objectweb.fractal.adl.arguments.ArgumentComponentLoader;
import org.objectweb.fractal.adl.attributes.AttributeLoader;
import org.objectweb.fractal.adl.bindings.TypeBindingLoader;
import org.objectweb.fractal.adl.implementations.ImplementationLoader;
import org.objectweb.fractal.adl.interfaces.InterfaceLoader;
import org.objectweb.fractal.adl.types.TypeLoader;
import org.objectweb.fractal.adl.xml.XMLLoader;

public class TestArguments extends Test {
  
  public Loader l;
  
  public Map context;
  
  public TestArguments (final String name) {
    super(name);
  }
  
  protected void setUp () {
    context = new HashMap();
    
    ArgumentLoader pv = new ArgumentLoader();
    InterfaceLoader iv = new InterfaceLoader();
    TypeLoader tv = new TypeLoader();
    ImplementationLoader impv = new ImplementationLoader();
    AttributeLoader attrv1 = new AttributeLoader();
    ArgumentComponentLoader compv = new ArgumentComponentLoader();
    TypeBindingLoader bindv = new TypeBindingLoader();
    AttributeLoader attrv2 = new AttributeLoader();
    
    pv.clientLoader = new XMLLoader(false);
    iv.clientLoader = pv;
    tv.clientLoader = iv;
    impv.clientLoader = tv;
    attrv1.clientLoader = impv;
    compv.clientLoader = attrv1;
    bindv.clientLoader = compv;
    attrv2.clientLoader = bindv;
    l = attrv2;
  }
  
  public void test1 () throws ADLException {
    context.put("p", "org.objectweb.fractal.adl.conform.components.IImpl");
    Definition d1 = l.load("ARG1", context);
    Definition d2 = l.load("ARG2", null);
    compare(d1, d2);
  }
  
  public void test2 () {
    try {
      l.load("ARG1", context);
      fail();
    } catch (ADLException e) {
    }
  }
  
  public void test3 () throws ADLException {
    context.put("p", "org.objectweb.fractal.adl.conform.components.IImpl");
    context.put("q", "ARG1");
    context.put("r", "primitive");
    Definition d1 = l.load("ARG3", context);
    Definition d2 = l.load("ARG4", null);
    compare(d1, d2);
  }
  
  public void test4 () throws ADLException {
    context.put("p", "org.objectweb.fractal.adl.conform.components.IImpl");
    context.put("q", "org.objectweb.fractal.adl.conform.components.IImpl");
    Definition d1 = l.load("ARG5", context);
    Definition d2 = l.load("ARG6", null);
    compare(d1, d2);
  }
  
  public void test5 () throws ADLException {
    context.put("u1", "org.objectweb.fractal.adl.conform.components.IImpl");
    context.put("v1", "org.objectweb.fractal.adl.conform.components.IImpl");
    context.put("u2", "org.objectweb.fractal.adl.conform.components.IImpl");
    context.put("v2", "org.objectweb.fractal.adl.conform.components.IImpl");
    Definition d1 = l.load("ARG7", context);
    Definition d2 = l.load("ARG8", null);
    compare(d1, d2);
  }
  
  public void test6 () {
    try {
      context.put("u", "org.objectweb.fractal.adl.conform.components.IImpl");
      l.load("ARG9", context);
      fail();
    } catch (ADLException e) {
    }
  }
  
  public void test7 () {
    try {
      context.put("u", "org.objectweb.fractal.adl.conform.components.IImpl");
      l.load("ARG10", context);
      fail();
    } catch (ADLException e) {
    }
  }
}
