/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.conform;

import java.util.HashSet;
import java.util.Set;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Loader;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.ComponentLoader;
import org.objectweb.fractal.adl.xml.XMLLoader;

public class TestSharing extends Test {
  
  public Loader l;
  
  public TestSharing (final String name) {
    super(name);
  }
  
  protected void setUp () {
    ComponentLoader compLoader = new ComponentLoader();
    compLoader.clientLoader = new XMLLoader(false);
    l = compLoader;
  }
  
  public void test1 () throws ADLException {
    ComponentContainer d = (ComponentContainer)l.load("SHARING1", null);
    assertEquals(1, getComponents(d, "a", new HashSet()).size());
    assertEquals(1, getComponents(d, "b", new HashSet()).size());
    assertEquals(1, getComponents(d, "c", new HashSet()).size());
    assertEquals(1, getComponents(d, "d", new HashSet()).size());
  }
  
  public void test2 () throws ADLException {
    ComponentContainer d = (ComponentContainer)l.load("SHARING2", null);
    assertEquals(1, getComponents(d, "a", new HashSet()).size());
    assertEquals(1, getComponents(d, "b", new HashSet()).size());
    assertEquals(1, getComponents(d, "c", new HashSet()).size());
    assertEquals(1, getComponents(d, "d", new HashSet()).size());
    assertEquals(1, getComponents(d, "e", new HashSet()).size());
  }
  
  public void test3 () throws ADLException {
    ComponentContainer d = (ComponentContainer)l.load("SHARING4", null);
    assertEquals(1, getComponents(d, "a", new HashSet()).size());
    assertEquals(1, getComponents(d, "b", new HashSet()).size());
  }

  public void test4 () throws ADLException {
    ComponentContainer d = (ComponentContainer)l.load("SHARING5", null);
    assertEquals(1, getComponents(d, "a", new HashSet()).size());
    assertEquals(1, getComponents(d, "b", new HashSet()).size());
  }

  public void test5 () throws ADLException {
    ComponentContainer d = (ComponentContainer)l.load("SHARING6", null);
    assertEquals(1, getComponents(d, "a", new HashSet()).size());
    assertEquals(1, getComponents(d, "b", new HashSet()).size());
  }

  private Set getComponents (final ComponentContainer c, final String n, final Set r) {
    if (c instanceof Component && ((Component)c).getName().equals(n)) {
      r.add(c);
    }
    Component[] comps = c.getComponents();
    for (int i = 0; i < comps.length; ++i) {
      getComponents(comps[i], n, r);
    }
    return r;
  }
}
